"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = exports.shorthands = exports.cliOptionsTypes = void 0;
exports.rcOptionsTypes = rcOptionsTypes;
exports.help = help;
exports.handler = handler;
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const config_1 = require("@pnpm/config");
const constants_1 = require("@pnpm/constants");
const plugin_commands_env_1 = require("@pnpm/plugin-commands-env");
const ci_info_1 = require("ci-info");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
const installDeps_1 = require("./installDeps");
function rcOptionsTypes() {
    return (0, pick_1.default)([
        'cache-dir',
        'child-concurrency',
        'dangerously-allow-all-builds',
        'dev',
        'engine-strict',
        'fetch-retries',
        'fetch-retry-factor',
        'fetch-retry-maxtimeout',
        'fetch-retry-mintimeout',
        'fetch-timeout',
        'frozen-lockfile',
        'global-dir',
        'global-pnpmfile',
        'global',
        'hoist',
        'hoist-pattern',
        'https-proxy',
        'ignore-pnpmfile',
        'ignore-scripts',
        'optimistic-repeat-install',
        'link-workspace-packages',
        'lockfile-dir',
        'lockfile-directory',
        'lockfile-only',
        'lockfile',
        'merge-git-branch-lockfiles',
        'merge-git-branch-lockfiles-branch-pattern',
        'modules-dir',
        'network-concurrency',
        'node-linker',
        'noproxy',
        'package-import-method',
        'pnpmfile',
        'prefer-frozen-lockfile',
        'prefer-offline',
        'production',
        'proxy',
        'public-hoist-pattern',
        'registry',
        'reporter',
        'save-workspace-protocol',
        'scripts-prepend-node-path',
        'shamefully-flatten',
        'shamefully-hoist',
        'shared-workspace-lockfile',
        'side-effects-cache-readonly',
        'side-effects-cache',
        'store-dir',
        'strict-peer-dependencies',
        'offline',
        'only',
        'optional',
        'unsafe-perm',
        'use-running-store-server',
        'use-store-server',
        'verify-store-integrity',
        'virtual-store-dir',
    ], config_1.types);
}
const cliOptionsTypes = () => ({
    ...rcOptionsTypes(),
    ...(0, pick_1.default)(['force'], config_1.types),
    'fix-lockfile': Boolean,
    'resolution-only': Boolean,
    recursive: Boolean,
});
exports.cliOptionsTypes = cliOptionsTypes;
exports.shorthands = {
    D: '--dev',
    P: '--production',
};
exports.commandNames = ['install', 'i'];
function help() {
    return (0, render_help_1.default)({
        aliases: ['i'],
        description: 'Installs all dependencies of the project in the current working directory. \
When executed inside a workspace, installs all dependencies of all projects.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Run installation recursively in every package found in subdirectories. \
For options that may be used with `-r`, see "pnpm help recursive"',
                        name: '--recursive',
                        shortAlias: '-r',
                    },
                    common_cli_options_help_1.OPTIONS.ignoreScripts,
                    common_cli_options_help_1.OPTIONS.offline,
                    common_cli_options_help_1.OPTIONS.preferOffline,
                    common_cli_options_help_1.OPTIONS.globalDir,
                    {
                        description: "Packages in `devDependencies` won't be installed",
                        name: '--prod',
                        shortAlias: '-P',
                    },
                    {
                        description: 'Only `devDependencies` are installed',
                        name: '--dev',
                        shortAlias: '-D',
                    },
                    {
                        description: 'Skip reinstall if the workspace state is up-to-date',
                        name: '--optimistic-repeat-install',
                    },
                    {
                        description: '`optionalDependencies` are not installed',
                        name: '--no-optional',
                    },
                    {
                        description: `Don't read or generate a \`${constants_1.WANTED_LOCKFILE}\` file`,
                        name: '--no-lockfile',
                    },
                    {
                        description: `Dependencies are not downloaded. Only \`${constants_1.WANTED_LOCKFILE}\` is updated`,
                        name: '--lockfile-only',
                    },
                    {
                        description: "Don't generate a lockfile and fail if an update is needed. This setting is on by default in CI environments, so use --no-frozen-lockfile if you need to disable it for some reason",
                        name: '--[no-]frozen-lockfile',
                    },
                    {
                        description: `If the available \`${constants_1.WANTED_LOCKFILE}\` satisfies the \`package.json\` then perform a headless installation`,
                        name: '--prefer-frozen-lockfile',
                    },
                    {
                        description: `The directory in which the ${constants_1.WANTED_LOCKFILE} of the package will be created. Several projects may share a single lockfile.`,
                        name: '--lockfile-dir <dir>',
                    },
                    {
                        description: 'Fix broken lockfile entries automatically',
                        name: '--fix-lockfile',
                    },
                    {
                        description: 'Merge lockfiles were generated on git branch',
                        name: '--merge-git-branch-lockfiles',
                    },
                    {
                        description: 'The directory in which dependencies will be installed (instead of node_modules)',
                        name: '--modules-dir <dir>',
                    },
                    {
                        description: 'Dependencies inside the modules directory will have access only to their listed dependencies',
                        name: '--no-hoist',
                    },
                    {
                        description: 'All the subdeps will be hoisted into the root node_modules. Your code will have access to them',
                        name: '--shamefully-hoist',
                    },
                    {
                        description: 'Hoist all dependencies matching the pattern to `node_modules/.pnpm/node_modules`. \
The default pattern is * and matches everything. Hoisted packages can be required \
by any dependencies, so it is an emulation of a flat node_modules',
                        name: '--hoist-pattern <pattern>',
                    },
                    {
                        description: 'Hoist all dependencies matching the pattern to the root of the modules directory',
                        name: '--public-hoist-pattern <pattern>',
                    },
                    common_cli_options_help_1.OPTIONS.storeDir,
                    common_cli_options_help_1.OPTIONS.virtualStoreDir,
                    {
                        description: 'Maximum number of concurrent network requests',
                        name: '--network-concurrency <number>',
                    },
                    {
                        description: 'Controls the number of child processes run parallelly to build node modules',
                        name: '--child-concurrency <number>',
                    },
                    {
                        description: 'Disable pnpm hooks defined in .pnpmfile.cjs',
                        name: '--ignore-pnpmfile',
                    },
                    {
                        description: 'Ignore pnpm-workspace.yaml if exists in the parent directory, and treat the installation as normal non-workspace installation.',
                        name: '--ignore-workspace',
                    },
                    {
                        description: "If false, doesn't check whether packages in the store were mutated",
                        name: '--[no-]verify-store-integrity',
                    },
                    {
                        description: 'Fail on missing or invalid peer dependencies',
                        name: '--strict-peer-dependencies',
                    },
                    {
                        description: 'Starts a store server in the background. The store server will keep running after installation is done. To stop the store server, run `pnpm server stop`',
                        name: '--use-store-server',
                    },
                    {
                        description: 'Only allows installation with a store server. If no store server is running, installation will fail',
                        name: '--use-running-store-server',
                    },
                    {
                        description: 'Clones/hardlinks or copies packages. The selected method depends from the file system',
                        name: '--package-import-method auto',
                    },
                    {
                        description: 'Hardlink packages from the store',
                        name: '--package-import-method hardlink',
                    },
                    {
                        description: 'Copy packages from the store',
                        name: '--package-import-method copy',
                    },
                    {
                        description: 'Clone (aka copy-on-write) packages from the store',
                        name: '--package-import-method clone',
                    },
                    {
                        description: 'Force reinstall dependencies: refetch packages modified in store, \
recreate a lockfile and/or modules directory created by a non-compatible version of pnpm. \
Install all optionalDependencies even they don\'t satisfy the current environment(cpu, os, arch)',
                        name: '--force',
                    },
                    {
                        description: 'Use or cache the results of (pre/post)install hooks',
                        name: '--side-effects-cache',
                    },
                    {
                        description: 'Only use the side effects cache if present, do not create it for new packages',
                        name: '--side-effects-cache-readonly',
                    },
                    {
                        description: 'Re-runs resolution: useful for printing out peer dependency issues',
                        name: '--resolution-only',
                    },
                    ...common_cli_options_help_1.UNIVERSAL_OPTIONS,
                ],
            },
            common_cli_options_help_1.OUTPUT_OPTIONS,
            common_cli_options_help_1.FILTERING,
        ],
        url: (0, cli_utils_1.docsUrl)('install'),
        usages: ['pnpm install [options]'],
    });
}
async function handler(opts) {
    const include = {
        dependencies: opts.production !== false,
        devDependencies: opts.dev !== false,
        optionalDependencies: opts.optional !== false,
    };
    // npm registry's abbreviated metadata currently does not contain libc
    // see <https://github.com/pnpm/pnpm/issues/7362#issuecomment-1971964689>
    const fetchFullMetadata = opts.rootProjectManifest?.pnpm?.supportedArchitectures?.libc && true;
    const installDepsOptions = {
        ...opts,
        frozenLockfileIfExists: opts.frozenLockfileIfExists ?? (ci_info_1.isCI && !opts.lockfileOnly &&
            typeof opts.rawLocalConfig['frozen-lockfile'] === 'undefined' &&
            typeof opts.rawLocalConfig['prefer-frozen-lockfile'] === 'undefined'),
        include,
        includeDirect: include,
        prepareExecutionEnv: plugin_commands_env_1.prepareExecutionEnv.bind(null, opts),
        fetchFullMetadata,
    };
    if (opts.resolutionOnly) {
        installDepsOptions.lockfileOnly = true;
        installDepsOptions.forceFullResolution = true;
    }
    return (0, installDeps_1.installDeps)(installDepsOptions, []);
}
//# sourceMappingURL=install.js.map